﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using NorthwindService;
using System.ServiceModel.Channels;

namespace ConsoleApp
{
    class Program
    {
        static void Main(string[] args)
        {
            var svc = new ChannelFactory<INorthwindService>("BasicHttpBinding_INorthwindService").CreateChannel();

            int successfulOps = 0;

            if (ValidSTEUpdate(svc))
            {
                successfulOps++;
            }
            Console.WriteLine("\n-----------------------------------------------\n");
            if (InvalidSTEUpdate(svc))
            {
                successfulOps++;
            }
            Console.WriteLine("\n-----------------------------------------------\n");
            if (DTOUpdate(svc))
            {
                successfulOps++;
            }

            Console.WriteLine("\n===============================================\n");
            Console.WriteLine(successfulOps + " of 3 operations performed as expected.\n");
            Console.ReadLine();
        }

        static bool ValidSTEUpdate(INorthwindService svc)
        {
            Console.WriteLine("Attempting VALID graph update with self-tracking entities:\n");

            var products = new List<Product>(svc.GetProducts());
            Console.WriteLine(products.Count + " products retrieved.");

            var customer = svc.GetCustomer("ALFKI");
            Console.Write("Customer retrieved: ");
            Write(customer);

            // modify contact name of customer
            customer.ContactName += "+";

            // remove all orders with a null order date
            foreach (var order in customer.Orders.Where(o => o.OrderDate == null).ToList())
            {
                customer.Orders.Remove(order);
            }

            // add a new order
            var newOrder = new Order();
            newOrder.Order_Details.Add(new Order_Detail() 
                { 
                    ProductID = products.Where(p => p.ProductName == "Chai")
                                        .Single().ProductID,
                    Quantity = 1 
                });
            customer.Orders.Add(newOrder);

            var submitSuccess = svc.SubmitOrder(newOrder);
            Console.WriteLine(submitSuccess ? "Saved." : "Didn't save.");

            customer = svc.GetCustomer("ALFKI");
            Console.Write("Updated customer retrieved: ");
            Write(customer);

            return submitSuccess;
        }

        static bool InvalidSTEUpdate(INorthwindService svc)
        {
            Console.WriteLine("Attempting INVALID graph update with self-tracking entitites:\n");
 
            var customer = svc.GetCustomer("ALFKI");
            var order = customer.Orders.First();
            order.MarkAsDeleted();

            bool submitSuccess = false;
            try
            {
                submitSuccess = svc.SubmitOrder(order);
            }
            catch (FaultException ex)
            {
                Console.WriteLine("Expected Failure: " + ex.Message);
            }

            // this operation expects the submit to fail, so return opposite
            return submitSuccess ? false : true;
        }

        static bool DTOUpdate(INorthwindService svc)
        {
            var customer = svc.GetCustomer("ALFKI");
            Write(customer);


            var submitSuccess = svc.SubmitOrderDTO(new NewOrderDTO
                {
                    CustomerID = customer.CustomerID,
                    ContactName = customer.ContactName + "-",
                    CustomerVersion = customer.Version,
                    Lines = new List<NewOrderLine>
                        {
                            new NewOrderLine { ProductID = 1, Quantity = 1 },
                        }
                });
            customer = svc.GetCustomer("ALFKI");
            Write(customer);

            return submitSuccess;
        }

        static void Write(Customer customer)
        {
            Console.WriteLine(customer.CustomerID + " - " + customer.ContactName);
            Console.WriteLine("\t" + customer.Orders.Count + " orders:");
            foreach (var order in customer.Orders)
            {
                Console.WriteLine("\t\t" + order.OrderID + " on " + order.OrderDate.ToString());
            }
        }
    }
}
